/* GCC */

#include <fstream>
#include <vector>
#include <list>

const int uninitialized = -1;

int findCycle(const std::vector<std::vector<int> >& graph,std::vector<bool>& visited,
              std::list<int>& answer,int current = 0,int parent = uninitialized)
{
    if(visited[current])
        return current;
    visited[current] = true;
    for(int i = 0;i < graph[current].size();i++)
    {
        if(graph[current][i] == parent)
            continue;
        int found = findCycle(graph,visited,answer,graph[current][i],current);
        if(found != uninitialized)
        {
            answer.push_back(current);
            if(found != current)
                return current;
            else
                break;
        }
    }
    return uninitialized;
}

void countForTree(const std::vector<std::vector<int> >& graph,std::vector<bool>& visited,int mod,
                  std::vector<int>& numberWith,std::vector<int>& numberWithout,int current,int parent = uninitialized)
{
    visited[current] = true;
    numberWith[current] = 0;
    numberWithout[current] = 1;
    std::vector<int> child;
    for(int i = 0;i < graph[current].size();i++)
    {
        if(graph[current][i] != parent && !visited[graph[current][i]])
        {
            child.push_back(graph[current][i]);
            countForTree(graph,visited,mod,numberWith,numberWithout,child.back(),current);
            numberWithout[current] = (numberWithout[current] *
                                      (numberWith[child.back()] + numberWithout[child.back()])) % mod;
        }
    }
    if(child.size() == 0)
        return;
    std::vector<int> prefix(child.size());
    std::vector<int> postfix(child.size());
    prefix.front() = (numberWith[child.front()] + numberWithout[child.front()]) % mod;
    for(int i = 1;i < prefix.size();i++)
        prefix[i] = (prefix[i-1] * ((numberWith[child[i]] + numberWithout[child[i]]) % mod)) % mod;
    postfix.back() = (numberWith[child.back()] + numberWithout[child.back()]) % mod;
    for(int i = postfix.size() - 2;i >= 0;i--)
        postfix[i] = (postfix[i+1] * ((numberWith[child[i]] + numberWithout[child[i]]) % mod)) % mod;
    for(int i = 0;i < child.size();i++)
    {
        int temp = 1;
        if(i > 0)
            temp = (temp * prefix[i-1]) % mod;
        if(i < postfix.size() - 1)
            temp = (temp * postfix[i+1]) % mod;
        temp = (temp * numberWithout[child[i]]) % mod;
        numberWith[current] = (numberWith[current] + temp) % mod;
    }
}

int count(const std::vector<std::vector<int> >& graph,int mod)
{
    int answer = 0;
    std::list<int> cycle;
    std::vector<bool> visited(graph.size());
    findCycle(graph,visited,cycle);
    visited.assign(graph.size(),false);
    for(std::list<int>::iterator i = cycle.begin();i != cycle.end();i++)
        visited[*i] = true;
    int temp = 1;
    std::vector<int> numberWith(graph.size());
    std::vector<int> numberWithout(graph.size());
    for(int i = 0;i < graph.size();i++)
    {
        if(visited[i])
            continue;
        countForTree(graph,visited,mod,numberWith,numberWithout,i);
        temp = (temp * (numberWith[i] + numberWithout[i])) % mod;
    }
    answer = (answer + 2*temp) % mod;
    visited.assign(graph.size(),false);
    numberWith.assign(graph.size(),0);
    numberWithout.assign(graph.size(),0);
    countForTree(graph,visited,mod,numberWith,numberWithout,cycle.front(),cycle.back());
    answer = (answer + (numberWith[cycle.front()] + numberWithout[cycle.front()]) % mod) % mod;
    visited.assign(graph.size(),false);
    numberWith.assign(graph.size(),0);
    numberWithout.assign(graph.size(),0);
    visited[cycle.front()] = true;
    visited[cycle.back()] = true;
    temp = 1;
    for(int i = 0;i < graph.size();i++)
    {
        if(visited[i])
            continue;
        countForTree(graph,visited,mod,numberWith,numberWithout,i);
        temp = (temp * (numberWith[i] + numberWithout[i])) % mod;
    }
    answer = (answer + temp) % mod;
    return answer;
}

int main()
{
    std::ifstream in("museum.in");
    int n,p;
    in >> n >> p;
    std::vector<std::vector<int> > graph(n);
    for(int i = 0;i < graph.size();i++)
    {
        int v,u;
        in >> v >> u;
        graph[v-1].push_back(u-1);
        graph[u-1].push_back(v-1);
    }
    in.close();
    std::ofstream out("museum.out");
    out << count(graph,p) << std::endl;
    out.close();
}
